;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x178f15d7573d91c4e875c0fdd50928d7 . "./code.cg2js/cg-acache.fasl")
(#xef657381ba7c0b3bd79763004be3594c . "./code.cg2js/cg-base.fasl")
(#x81e09fb66c2a0476349662062bc64c32 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xba6a7c541b6995791f69340d4d62e854 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x36fedee5a136d723c5240dd7f684b2d9 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xfb286535a117ef77b04a84e2880d9193 . "./code.cg2js/cg-button.fasl")
(#x04b9de1864ecd5709bdfa7b28087ffed . "./code.cg2js/cg-calendar.fasl")
(#x14edfadb58812a0aadf47ddcea6f35e7 . "./code.cg2js/cg-caret.fasl")
(#xe16bf439b12775798ec93ab03d8f6233 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xcaa27a599da62d9dc07487aa64082a2c . "./code.cg2js/cg-chart-widget.fasl")
(#x5eace84e031b1136a83ee41e237bcd53 . "./code.cg2js/cg-check-box.fasl")
(#xf065079eac89e083847107895b4a9992 . "./code.cg2js/cg-choice-list.fasl")
(#x41b13536bb1e275996da55cd585238be . "./code.cg2js/cg-choose-printer.fasl")
(#x53e36ec2840275729d3e11b151b8d103 . "./code.cg2js/cg-class-grid.fasl")
(#x589458736367c37a8e5a25789cf80438 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x369b80b53558adb7bcb439544bf576b7 . "./code.cg2js/cg-class-support.fasl")
(#xa082ca12075131287fe50153f87bea50 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x7367f28a7ab2e1dc50b353d9ffd0006a . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1c0647f9c728646ab5db7f471378365c . "./code.cg2js/cg-clipboard.fasl")
(#x316661f687c051c8f8cce881acde5a92 . "./code.cg2js/cg-color-dialog.fasl")
(#xfd82de29f80647c063cf117f3013bd75 . "./code.cg2js/cg-combo-box.fasl")
(#x3a14f20c84bdfc7bc5bba4afb8d63d66 . "./code.cg2js/cg-common-control.fasl")
(#xc9489e1cdaadf9cbf819a8940a5814f7 . "./code.cg2js/cg-comtab.fasl")
(#x673b58fbdba54a648a21bd2b518146a5 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x497f3305891725ea67b708f0a0e60af5 . "./code.cg2js/cg-curve.fasl")
(#xe258d469256acf5f344a7e84e2414e2d . "./code.cg2js/cg-dde-utils.fasl")
(#x6a19ccd0351eef4eff1e78393dbce5d3 . "./code.cg2js/cg-dev.fasl")
(#x3ed6667ca9cfaee9242cb1a9a65684d3 . "./code.cg2js/cg-dialog-item.fasl")
(#x2351a5306cf92a668179319acab833c7 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xdc0260d39ef0b773d4332029189dc279 . "./code.cg2js/cg-directory-dialog.fasl")
(#x22a3a9257553da0f824237844ddd1dee . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x69923db2fc04b1f64281982794c436bc . "./code.cg2js/cg-drag-and-drop.fasl")
(#x8785ed34bc744ef5ddcc0681734bf048 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x78391bf518ecab11576dccfe66f2556a . "./code.cg2js/cg-drawable.fasl")
(#x3aff80b82ee8f422932654fe4c61c6c4 . "./code.cg2js/cg-dropping-outline.fasl")
(#xb2a1abfff80ef3fd558adaa5093f9c2f . "./code.cg2js/cg-edit-in-place.fasl")
(#xd86aa3722249badcbba174a8909b1c38 . "./code.cg2js/cg-editable-text.fasl")
(#x4751cb09c766716e14dd4970791d6d94 . "./code.cg2js/cg-file-dialog.fasl")
(#xd0c44476ef07391addfb1f299dee6be9 . "./code.cg2js/cg-fill-texture.fasl")
(#xa8a407e8913a76aba4e7abd39e63312a . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2c87e2e8508013ee7ab793e891ea7f69 . "./code.cg2js/cg-font-dialog.fasl")
(#xde1b0ac556b85c1b68ae2f03510989b9 . "./code.cg2js/cg-gesture-emulation.fasl")
(#xf75f0cc3c746697153656ec947ec9d59 . "./code.cg2js/cg-get-pixmap.fasl")
(#x3b347f78a076d8f2b8325a0150e258f5 . "./code.cg2js/cg-get-position.fasl")
(#x7bdfb1bdc7cb744ae4580cd4cecb3100 . "./code.cg2js/cg-graphics-context.fasl")
(#x45bfd4964591adef120d1b41a32fef4e . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x5dab568bf3bd82b31c5e2d7981baa08e . "./code.cg2js/cg-grid-widget.fasl")
(#xbf2c64d864c17820b372905092ddeefe . "./code.cg2js/cg-group-box.fasl")
(#xf712467e73b99a00808ab6d01e1b4227 . "./code.cg2js/cg-header-control.fasl")
(#x2fa94f0a05fdc4f4b5ccbcf9f595e4a6 . "./code.cg2js/cg-hotspot.fasl")
(#xe66bc4fcbeab3d45f62110b64042e212 . "./code.cg2js/cg-html-dialog.fasl")
(#x94d8798a47256dfb9b3459a512eec6f2 . "./code.cg2js/cg-html-widget.fasl")
(#xde295023d4b89e1fe000ba4f842c9d08 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xd78bdfa404ca828c519ced2eef450b7f . "./code.cg2js/cg-icon.fasl")
(#x29286f4fbafee33314db6eebde50d5df . "./code.cg2js/cg-ie.fasl")
(#xa8e1d3ce6cbf1fbb6573f0e86a49f187 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x75554c9befce5e533be4493a3c9456a5 . "./code.cg2js/cg-intersect.fasl")
(#x5de5174ca9a5c2f4e47246e8b9ea0e43 . "./code.cg2js/cg-item-list.fasl")
(#xb919f869db3c7fcf3a0bc41129bb495c . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xf893cecda67e75bc762a25f04897865d . "./code.cg2js/cg-lamp.fasl")
(#x4d5a14784d1094aec7534b29ba7331db . "./code.cg2js/cg-layout.fasl")
(#x6e3cbd84078e8dbbd70cb979afe288ea . "./code.cg2js/cg-lettered-menu.fasl")
(#x588c04c5489c77f838dbfa3f5b495e2e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x36d5b7bd5d87ba1577688f30bf4a142e . "./code.cg2js/cg-lisp-text.fasl")
(#x3a36095434c01653995782f9a77927f1 . "./code.cg2js/cg-lisp-widget.fasl")
(#xa4f50da79e8464a1f492efcde4e30b37 . "./code.cg2js/cg-list-view.fasl")
(#x897b3ec7f0442e71e9b57531f0f6ff49 . "./code.cg2js/cg-mci.fasl")
(#xe5a3ea098bdee08bb577196a2de58f4e . "./code.cg2js/cg-menu-tooltip.fasl")
(#xc469d63651a919d6d13ad9232420883b . "./code.cg2js/cg-menu.fasl")
(#xab168d71e790e19dca9aaab3f18ed9b9 . "./code.cg2js/cg-message-dialog.fasl")
(#xfdeb7418bae6164d6ad3c9762468b232 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x2bafa0c542deba6bf4e0786c8aa705b1 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x6d8f1c8e1a6abc45aa3c40e0c724e419 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xf39aef8a2bdc89ef2ff297210dd51814 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x64fa12ee36b6a8a677b64533ac27fc65 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x2245f0efbd8e8e8e55816d2f8c6cca08 . "./code.cg2js/cg-nodes.fasl")
(#x5d659d9b4cc6bc5215f5c3902814aba1 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x6ae5b7072a222f1f903ebe23ced394d4 . "./code.cg2js/cg-object-editor.fasl")
(#x95894e820344494f041e1a160724392d . "./code.cg2js/cg-ocx.fasl")
(#x1aaf80f173f255c2087523827f99f6be . "./code.cg2js/cg-os-widget.fasl")
(#xb23ebfa5e6ecaf4e63efc30ca0e481d9 . "./code.cg2js/cg-os-window.fasl")
(#xfbc0f0b573aefeab188d0dd451084c66 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x28e559ee300142543b8537ebd16b7e01 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc8678f6b46fdfacac6a5f9cd0f762d4e . "./code.cg2js/cg-outline.fasl")
(#xe6b46d375e97ef79d86e2a7cf89f47aa . "./code.cg2js/cg-palette.fasl")
(#x32d0034bea27470caaff0bb99373946f . "./code.cg2js/cg-paren-matching.fasl")
(#x9c2212e27d9a85622a6ef00cd6804d85 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xbee7a0d5b15be8c63dd7b7acf009afa4 . "./code.cg2js/cg-picture-widget.fasl")
(#x213dc95fbedd8a66f674848f2b11e825 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xca95dc74d1edbd7ada8622f4630ace2d . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0398011748557bb941420c570ff53dec . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x5302c47124ced5a31e102ffba2731987 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xe3a077b1fadf6335406cb4dba931beb3 . "./code.cg2js/cg-pixmap.fasl")
(#x82de699ff0a7c2493cf4f01ba1b6a587 . "./code.cg2js/cg-player.fasl")
(#x32e876bd033f166c92475a3268cdead3 . "./code.cg2js/cg-plot-widget.fasl")
(#x160933371462646c4c5abc3bfdaa1011 . "./code.cg2js/cg-printing.fasl")
(#xa5509885660c8508ff968f88b928a5c6 . "./code.cg2js/cg-progress-indicator.fasl")
(#x233902229a0e55fd7502dff830d8994b . "./code.cg2js/cg-project-window.fasl")
(#x4abf02bf1bc97cd3d4d500e995f93f11 . "./code.cg2js/cg-property.fasl")
(#xc16ea32cdcab1440cff31a0528f5eca8 . "./code.cg2js/cg-radio-button.fasl")
(#xa4be9fe7cb2fe3917c345244e37ecb87 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x09e93a413869d2229b5667e6b8edb43a . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x7226ec1f8382f0fc880fd8fe976601e0 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xbd57654b22fb8142de77828564f9caaf . "./code.cg2js/cg-rich-edit.fasl")
(#x3ad2054412cc4bb464fb635bdff3d79f . "./code.cg2js/cg-sample-file-menu.fasl")
(#x2e314fbab15e136080d8ed51df25b1db . "./code.cg2js/cg-scaling-stream.fasl")
(#x9859adc5af1ee19e8c6526310909ee90 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xc18d82717c354e62787827afc20d724b . "./code.cg2js/cg-scroll-bar.fasl")
(#x03568b75b47407955970f496823ee14c . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x93458c9923d1aeb1807b020ecef98ee0 . "./code.cg2js/cg-selected-object.fasl")
(#xc207f3a29eb6c3f8b54ae4ee66927dc3 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x8e0b1519633147530035c8d02752a683 . "./code.cg2js/cg-split-bar.fasl")
(#xaf34d74929078ea03ee9847d3650f3f7 . "./code.cg2js/cg-static-text.fasl")
(#xd5a3440e3e2edc5982f7363ad5f3b024 . "./code.cg2js/cg-status-bar.fasl")
(#xbbf36d08e674d8883d011d32837d2d3c . "./code.cg2js/cg-string-dialog.fasl")
(#xa6c4bc62d489ef503cc3259a520b2524 . "./code.cg2js/cg-tab-control.fasl")
(#xad3ac73dce7621c01c8a4273028ca501 . "./code.cg2js/cg-template-string.fasl")
(#xc0a8a07d55dd134d211dcc1099a4c404 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xdac9e3cd2224ce26453df9f3cc720336 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xd8d0fb5ad769746ee4fa08fccf3c49d5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xa5c72c82461f2c10f0db9a777d102caa . "./code.cg2js/cg-text-or-combo.fasl")
(#x9ebcfb9a9d221a922c6e161a6852fc64 . "./code.cg2js/cg-text-widget.fasl")
(#x66302590919481605337261b1c88ad03 . "./code.cg2js/cg-timer.fasl")
(#x8b5159f6d8bfc89a4948a3d34e9f774e . "./code.cg2js/cg-toggling-widget.fasl")
(#xca987e1619fc404fd382192e37d2db72 . "./code.cg2js/cg-toolbar.fasl")
(#xd37703c868941af7e5645528fd42f1f1 . "./code.cg2js/cg-tooltip.fasl")
(#x6b17da39b30ea81ec34c65d41274c454 . "./code.cg2js/cg-touch.fasl")
(#x4be7b27fb4ea6d64a716ba67f729cc63 . "./code.cg2js/cg-trackbar.fasl")
(#x8e112b9745a3ce23356ca040cb3280f3 . "./code.cg2js/cg-tray.fasl")
(#xb651a3fc232584ec80fa8ce4b8ebbc2a . "./code.cg2js/cg-up-down-control.fasl")
(#x00a8a0eca10a58444f41f8f40e6146dd . "./code.cg2js/cg-utility-dialog.fasl")
(#xb3b75ce875ee22ca8d022cb7be3334dd . "./code.cg2js/cg-web-browser-dde.fasl")
(#x2d3dedec29b45df71eae5d75b8ad82d1 . "./code.cg2js/cg-web-browser.fasl")
(#x9756708758b066242945a1814c69b792 . "./code.cg2js/cg-wrap-string.fasl")
(#xfc62c9d54238d7824e72bfb74b56298b . "./code.cg2js/cg-yes-no-list.fasl")
(#x256f74249573a5ade5c8dc85ec807537 . "./code.cg2js/cg.fasl")
(#xdc61e2b1b82270e01442ecb5cfbc9387 . "./code.cg2js/cg-yes-no-string.fasl")
(#x77a2ece0445c762d30c129dc4f98aa38 . "./code.idejs/ide-acache.fasl")
(#x3d1aeb180a1784f047d42011cf6b5b73 . "./code.idejs/ide-apropos.fasl")
(#x0d2deb2a3505e09e809f280e1ee04c46 . "./code.idejs/ide-base.fasl")
(#x3870eae654c0d92c6ecb5bf7f42d901b . "./code.idejs/ide-class-browser.fasl")
(#x288a551a81ecb0fa8bd14d7c6112c281 . "./code.idejs/ide-class-grapher.fasl")
(#x54c970ea70cab8dc0946977db20afb4d . "./code.idejs/ide-clipboard.fasl")
(#xa5b814fc37e935626134e1010c178cf8 . "./code.idejs/ide-dev.fasl")
(#xd49bee320fb8b039069c52f18d6f6a7a . "./code.idejs/ide-editor.fasl")
(#xf7c4e8af067c447e3d1789416c58a4ce . "./code.idejs/ide-find-definitions.fasl")
(#xbc88a1675504d2823fea60bfe10ba1c1 . "./code.idejs/ide-find-in-files.fasl")
(#xe9f061cfdba129d3d4f855e0e6b65fc6 . "./code.idejs/ide-form.fasl")
(#x1b17548f862c4836e9acef221b5e296a . "./code.idejs/ide-grapher.fasl")
(#xd0cd6375a3caec2190b5d623ef7ae407 . "./code.idejs/ide-inspector.fasl")
(#x44da39b5245bf3cf931cb5c362dd1141 . "./code.idejs/ide-intro.fasl")
(#x95e5c3eecff486b2c15f050ace553add . "./code.idejs/ide-keyshort.fasl")
(#x114796593237d91e67305f282a8874cf . "./code.idejs/ide-navigator.fasl")
(#xfe39368748ec2654f9e5bfcd0583f7ff . "./code.idejs/ide-package-list.fasl")
(#x871644b51f9b3622f993f66a96475a3b . "./code.idejs/ide-patch-dialog.fasl")
(#xd590ad2e9dcb98e7092848536577cf22 . "./code.idejs/ide-process-browser.fasl")
(#x1b603437fb4a01f85710b7288d3ddbfa . "./code.idejs/ide-project.fasl")
(#xd43e232207b8edbc4b9bf20bb427db40 . "./code.idejs/ide-trace.fasl")
(#x322927c6f53fa5a6a37a4d87138cc720 . "./code.idejs/ide-window-list.fasl")
(#xe1ccbe678cb7c55e37407fa65c70add8 . "./code.idejs/ide.fasl")
(#xf2773f648685397c1e60fb942db24e35 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
